@@ -1,4 +1,4 @@
-/* dataio.c   by Michael Thorpe   2004-06-19 */
+/* dataio.c   by Michael Thorpe   2004-06-23 */
 
 #include <errno.h>
 #include <stdlib.h>
@@ -45,13 +45,13 @@ int dconnect(conn *c) {
       return(1);
    j=sizeof(i);
    if(getsockopt(c->fdd,SOL_SOCKET,SO_ERROR,&i,&j) || j != sizeof(i)) {
-      derror(c);
+      dclose(c,425);
       return(1);
    }
-   if(i==EINPROGRESS)
-      return(0);
    if(i) {
-      derror(c);
+      if(i==EINPROGRESS)
+         return(0);
+      dclose(c,425);
       return(1);
    }
    c->dstate=DSTATE_OPEN;
@@ -92,8 +92,8 @@ int dwriterec(conn *c,char *buf,size_t len) {
    return(0);
 }
 
-int dflush(conn *c) {
-   if(c->doutbuflen)
+int dflush(conn *c,int abort) {
+   if(!abort && c->doutbuflen)
       return(dout(c));
    c->callback=0;
    return(dclose(c,0));
@@ -110,8 +110,7 @@ tryagain:
          if(i==-1 && errno==EINTR)
             goto tryagain;
          if(i==-1 && errno != EAGAIN) {
-/* FIXME: Nearly all instances of derror() are highly suspect */
-            derror(c);
+            dclose(c,426);
             return(1);
          }
       } else if(i<c->doutbuflen) {
@@ -180,6 +179,8 @@ int dopen(conn *c,int direction) {
 int dclose(conn *c,int failure) {
    int i=0;
 
+   if(c->callback)
+      (c->callback)(c,1);
    switch(c->dstate) {
    case DSTATE_CLOSED:
       return(0);
@@ -209,8 +210,3 @@ int dclose(conn *c,int failure) {
    return(i);
 }
 
-int derror(conn *c) {
-/* FIXME: Needs to be written */
-   return(1);
-}
-
