@@ -1,4 +1,4 @@
-/* file.c   by Michael Thorpe   2004-06-18 */
+/* file.c   by Michael Thorpe   2004-06-23 */
 
 #include <errno.h>
 #include <fcntl.h>
@@ -14,13 +14,13 @@
 #error "MAX_OUT_BUF must not be smaller than BUF_SIZE!"
 #endif
 
-static int store_cb(conn *c) {
+static int store_cb(conn *c,int abort) {
    char *buf;
    size_t l;
    int z;
 
    z=451;
-   if(c->state>=STATE_ABORT)
+   if(abort || c->state>=STATE_ABORT)
       goto fail;
    buf=(char *)malloc(BUF_SIZE);
    if(!buf)
@@ -86,12 +86,12 @@ int cmd_dele(conn *c,char *arg) {
    return(putline(c,250,0));
 }
 
-static int retr_cb(conn *c) {
+static int retr_cb(conn *c,int abort) {
    char *buf;
    size_t l;
    int z=451;
 
-   if(c->state>=STATE_ABORT)
+   if(abort || c->state>=STATE_ABORT)
       goto fail;
    while(davail(c,2*BUF_SIZE)) {
       buf=(char *)malloc(2*BUF_SIZE);
@@ -122,7 +122,7 @@ fail:
       return(dclose(c,z));
    }
    c->callback=dflush;
-   return(dflush(c));
+   return(dflush(c,0));
 }
 
 int cmd_retr(conn *c,char *arg) {
