@@ -1,4 +1,4 @@
-/* main.c   by Michael Thorpe   2002-12-27 */
+/* main.c   by Michael Thorpe   2004-06-18 */
 
 #include <errno.h>
 #include <fcntl.h>
@@ -60,8 +60,8 @@ int main(int argc,char **argv) {
 
    if(argc>1)
       port=strtoul(argv[1],0,10);
-   if(0==port || argc>3) {
-      printf("usage: ftpd [<port> [<addr>]]\n");
+   if(0==port || argc>4) {
+      printf("usage: ftpd [<port> [<addr> [<external_addr>]]]\n");
       return(1);
    }
 #ifndef DEBUG
@@ -69,7 +69,7 @@ int main(int argc,char **argv) {
    close(1);
 #endif
    curpid=getpid();
-   if(newlistener(argc==3?argv[2]:0,port)) {
+   if(newlistener(argc>=3?argv[2]:0,port,argc>=4?argv[3]:0)) {
       fputs("Couldn't listen!\n",stderr);
       return(1);
    }
