@@ -1,4 +1,4 @@
-/* net.c   by Michael Thorpe   2002-12-27 */
+/* net.c   by Michael Thorpe   2004-06-18 */
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -133,7 +133,8 @@ conn *newconn(int fd,struct sockaddr_in *sar,listener *l) {
    c->gid=BASE_GID;
    c->fdc=fd;
    c->fdd=-1;
-   c->dstate=DSTATE_ACTIVE;
+   c->dmode=DMODE_ACTIVE;
+   c->dstate=DSTATE_CLOSED;
    unsettimeout(c);
    c->mode='S';
    c->type='A';
@@ -156,6 +157,7 @@ conn *newconn(int fd,struct sockaddr_in *sar,listener *l) {
    c->drport=ntohs(sar->sin_port);
    if(l && l->addr) {
       c->laddr=strdup(l->addr);
+      c->external_laddr=strdup(l->external_addr?l->external_addr:l->addr);
       c->lport=l->cport;
    } else {
       i=sizeof(sa);
@@ -164,6 +166,7 @@ conn *newconn(int fd,struct sockaddr_in *sar,listener *l) {
          goto fail;
       }
       c->laddr=strdup(inet_ntoa(sa.sin_addr));
+      c->external_laddr=strdup(c->laddr);
       c->lport=ntohs(sa.sin_port);
    }
    c->dlport=c->lport-1;
@@ -227,7 +230,7 @@ void delconn(conn *c) {
    dropuserid();
 }
 
-int newlistener(const char *addr,unsigned short port) {
+int newlistener(const char *addr,unsigned short port,const char *external_addr) {
    listener *l;
    char tmp[22];
 
@@ -254,6 +257,14 @@ int newlistener(const char *addr,unsigned short port) {
    }
    if(log2(0,LOG_NOTICE,"Listening on ",tmp))
       goto failclose;
+   if(external_addr) {
+      l->external_addr=strdup(external_addr);
+      if(!l->external_addr) {
+         log(0,LOG_FAIL,"Couldn't allocate memory");
+         goto failclose;
+      }
+   } else
+      l->external_addr=0;
    if(addr) {
       l->addr=strdup(addr);
       if(!l->addr) {
@@ -359,7 +370,13 @@ int cmd_pasv(conn *c,char *arg) {
    struct sockaddr_in sa;
    char buf[49];
 
-   s=bindaddrport(c->laddr,0);
+#define USE_PORT20_FOR_PASV 1
+   if(-1 != c->fdd) {
+      i=dclose(c,0);
+      if(i)
+         return(i);
+   }
+   s=bindaddrport(c->laddr,USE_PORT20_FOR_PASV?20:0);
    if(s<0)
       return(1);
    i=sizeof(sa);
@@ -376,8 +393,9 @@ int cmd_pasv(conn *c,char *arg) {
    port=ntohs(sa.sin_port);
    c->dlport=port;
    c->fdd=s;
-   c->dstate=DSTATE_PASSIVE;
-   sprintf(buf,"Entering Passive Mode (%s,%u,%u).",c->laddr,port>>8,port&255);
+   c->dmode=DMODE_PASSIVE;
+   c->dstate=DSTATE_OPENING;
+   sprintf(buf,"Entering Passive Mode (%s,%u,%u).",c->external_laddr?c->external_laddr:c->laddr,port>>8,port&255);
    for(s=24;buf[s] != ',';s++)
       if(buf[s]=='.')
          buf[s++]=',';
